@@ -33,6 +33,8 @@
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
 
+#include <libxfsm/xfsm-util.h>
+
 #include "xfce4-session-settings-common.h"
 #include "xfce4-session-marshal.h"
 #include "xfsm-client-dbus-client.h"
@@ -73,7 +75,7 @@ session_editor_ensure_dbus(void)
 {
     GError *error = NULL;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     if (manager_dbus_proxy)
         return TRUE;
@@ -113,14 +115,21 @@ pulse_session_save_dialog(gpointer data)
 
 static void
 session_editor_save_session(GtkWidget *btn,
-                            GtkWidget *dialog)
+                            GtkBuilder *builder)
 {
+    GtkWidget *dialog = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_saving"));
+    GtkWidget *treeview = GTK_WIDGET (gtk_builder_get_object (builder, "saved-sessions-list"));
+    GtkWidget *notebook = GTK_WIDGET (gtk_builder_get_object (builder, "plug-child"));
+    GtkWidget *btn_clear = GTK_WIDGET (gtk_builder_get_object(builder, "btn_clear_sessions"));
     GtkWidget *pbar = g_object_get_data(G_OBJECT(dialog), "pbar");
+    GtkTreeModel    *model;
+    GList *sessions;
+    XfceRc *rc;
     guint pulse_id;
     guint sig_id;
     GError *error = NULL;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     gtk_widget_set_sensitive(btn, FALSE);
 
@@ -147,6 +156,23 @@ session_editor_save_session(GtkWidget *btn,
     g_signal_handler_disconnect(manager_dbus_proxy, sig_id);
     gtk_widget_hide(dialog);
     gtk_widget_set_sensitive(btn, TRUE);
+    /* After saving the session we ensure the clear button is sensitive */
+    gtk_widget_set_sensitive (btn_clear, TRUE);
+    /* Always make sure the "Saved Sessions" tab is visible  and the treeview is populated after saving a session */
+    rc = settings_list_sessions_open_rc ();
+    if (rc)
+    {
+        gtk_widget_show (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 3));
+        sessions = settings_list_sessions (rc);
+        model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
+        /* If the treeview hasn't been initialized we do it now */
+        if (!GTK_IS_LIST_STORE (model))
+        {
+            settings_list_sessions_treeview_init (GTK_TREE_VIEW (treeview));
+            model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
+        }
+        settings_list_sessions_populate (model, sessions);
+    }
 }
 
 static void
@@ -160,9 +186,11 @@ session_editor_sel_changed_btn(GtkTreeSelection *sel,
 
 static void
 session_editor_clear_sessions(GtkWidget *btn,
-                              GtkWidget *treeview)
+                              GtkBuilder *builder)
 {
-    TRACE("entering");
+    GtkWidget *treeview = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_clients"));
+    GtkWidget *notebook = GTK_WIDGET (gtk_builder_get_object (builder, "plug-child"));
+    TRACE("entering\n");
 
     gtk_widget_set_sensitive(btn, FALSE);
 
@@ -218,6 +246,9 @@ session_editor_clear_sessions(GtkWidget *btn,
 
         g_dir_close(cache_dir);
         g_free(cache_dir_path);
+
+        /* Always make sure the "Saved Sessions" tab is hidden after deleting all sessions */
+        gtk_widget_hide (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 3));
     }
     else {
         gtk_widget_set_sensitive(btn, TRUE);
@@ -236,7 +267,7 @@ session_editor_quit_client(GtkWidget *btn,
     guchar hint = SmRestartIfRunning;
     gchar *primary;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
     if(!gtk_tree_selection_get_selected(sel, &model, &iter))
@@ -305,7 +336,7 @@ session_editor_set_from_desktop_file(GtkTreeModel *model,
     XfceRc *rcfile;
     const gchar *name, *icon;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     rcfile = xfce_rc_simple_open(desktop_file, TRUE);
     if(!rcfile)
@@ -350,7 +381,7 @@ client_sm_property_changed(XfsmClient *proxy,
     GtkTreeIter iter;
     gboolean has_desktop_file = FALSE;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     if(!gtk_tree_model_get_iter(model, &iter, path)) {
         gtk_tree_path_free(path);
@@ -400,7 +431,7 @@ client_state_changed(XfsmClient *proxy,
 {
     GtkTreeView *treeview = user_data;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     if(new_state == 7) {  /* disconnected.  FIXME: enum this */
         GtkTreeModel *model = gtk_tree_view_get_model(treeview);
@@ -439,7 +470,7 @@ manager_client_registered(XfsmManager *proxy,
     gchar *property;
     GError *error = NULL;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     DBG("new client at %s", object_path);
 
@@ -490,7 +521,7 @@ manager_client_registered(XfsmManager *proxy,
                 priority = g_variant_get_byte(variant_value);
                 DBG("priority %d", priority);
             }
-            else if(g_strcmp0(property, SmProgram) == 0)
+            else if(g_strcmp0(property, SmProcessID) == 0)
             {
                 pid = g_variant_get_string(variant_value, 0);
                 DBG("pid %s", pid);
@@ -520,9 +551,15 @@ manager_client_registered(XfsmManager *proxy,
                        COL_PID, pid,
                        -1);
 
-    if(desktop_file != NULL)
+    if (desktop_file != NULL)
     {
-        session_editor_set_from_desktop_file(model, &iter, desktop_file);
+        session_editor_set_from_desktop_file (model, &iter, desktop_file);
+    }
+    else if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (), name))
+    {
+        gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+                            COL_ICON_NAME, name,
+                            -1);
     }
 
     path = gtk_tree_model_get_path(model, &iter);
@@ -549,7 +586,7 @@ session_editor_create_restart_style_combo_model(void)
     GtkTreeIter iter;
     gint i;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     for(i = 0; restart_styles[i]; ++i) {
         gtk_list_store_append(ls, &iter);
@@ -570,7 +607,7 @@ priority_changed(GtkCellRenderer *render,
     GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
     GtkTreeIter iter;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     if(gtk_tree_model_get_iter(model, &iter, path)) {
         XfsmClient *proxy = NULL;
@@ -627,7 +664,7 @@ restart_style_hint_changed(GtkCellRenderer *render,
     GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
     GtkTreeIter iter;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     if(gtk_tree_model_get_iter(model, &iter, path)) {
         gint i;
@@ -680,7 +717,7 @@ session_tree_compare_iter(GtkTreeModel *model,
 {
     guchar aprio = 0, bprio = 0;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     gtk_tree_model_get(model, a, COL_PRIORITY, &aprio, -1);
     gtk_tree_model_get(model, b, COL_PRIORITY, &bprio, -1);
@@ -723,7 +760,7 @@ session_editor_populate_treeview(GtkTreeView *treeview)
     guint i;
     GError *error = NULL;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     render = gtk_cell_renderer_text_new();
     g_object_set(render,
@@ -821,7 +858,7 @@ session_editor_init(GtkBuilder *builder)
     GtkTreeView *treeview;
     GtkTreeSelection *sel;
 
-    TRACE("entering");
+    TRACE("entering\n");
 
     treeview = GTK_TREE_VIEW(gtk_builder_get_object(builder, "treeview_clients"));
     sel = gtk_tree_view_get_selection(treeview);
@@ -833,11 +870,11 @@ session_editor_init(GtkBuilder *builder)
 
     btn_save = gtk_builder_get_object(builder, "btn_save_session");
     g_signal_connect(btn_save, "clicked",
-                     G_CALLBACK(session_editor_save_session), GTK_WIDGET(dlg_saving));
+                     G_CALLBACK(session_editor_save_session), builder);
 
     btn_clear = gtk_builder_get_object(builder, "btn_clear_sessions");
     g_signal_connect(btn_clear, "clicked",
-                     G_CALLBACK(session_editor_clear_sessions), treeview);
+                     G_CALLBACK(session_editor_clear_sessions), builder);
 
     btn_quit = gtk_builder_get_object(builder, "btn_quit_client");
     g_signal_connect(btn_quit, "clicked",
