@@ -70,6 +70,7 @@ static void       xfsm_logout_dialog_finalize (GObject          *object);
 static GtkWidget *xfsm_logout_dialog_button   (const gchar      *title,
                                                const gchar      *icon_name,
                                                const gchar      *icon_name_fallback,
+                                               const gchar      *icon_name_fallback2,
                                                XfsmShutdownType  type,
                                                XfsmLogoutDialog *dialog);
 
@@ -161,6 +162,9 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
   gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+  /* Use Adwaita's keycap style to get a meaningful look out of the box */
+  gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (dialog)), "keycap");
+  gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (dialog)), "xfsm-logout-dialog");
 
   /* load xfconf settings */
   channel = xfsm_open_config ();
@@ -169,7 +173,7 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
 
   main_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, BORDER);
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), main_vbox, TRUE, TRUE, 0);
-  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), BORDER);
+  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
   gtk_widget_show (main_vbox);
 
   /* label showing the users' name */
@@ -182,6 +186,7 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
   label_str = g_strdup_printf (_("Log out %s"), username);
   label = gtk_label_new (label_str);
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_CENTER);
+  gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (label)), "xfsm-logout-label");
   gtk_box_pack_start (GTK_BOX (main_vbox), label, FALSE, TRUE, 0);
   gtk_widget_show (label);
   g_free (label_str);
@@ -211,19 +216,21 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
 
   button_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, BORDER);
   gtk_box_pack_start (GTK_BOX (vbox), button_vbox, FALSE, TRUE, 0);
+  gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (button_vbox)), "xfsm-logout-buttons");
   gtk_widget_show (button_vbox);
 
   /* row for logout/shutdown and reboot */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, BORDER);
   gtk_box_pack_start (GTK_BOX (button_vbox), hbox, FALSE, TRUE, 0);
+  gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);
   gtk_widget_show (hbox);
 
   /**
    * Logout
    **/
   button = xfsm_logout_dialog_button (_("_Log Out"), "system-log-out",
-                                      "xfsm-logout", XFSM_SHUTDOWN_LOGOUT,
-                                      dialog);
+                                      "xfsm-logout", NULL,
+                                      XFSM_SHUTDOWN_LOGOUT, dialog);
 
   gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
   gtk_widget_show (button);
@@ -242,8 +249,8 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
     }
 
   button = xfsm_logout_dialog_button (_("_Restart"), "system-reboot",
-                                      "xfsm-reboot", XFSM_SHUTDOWN_RESTART,
-                                      dialog);
+                                      "xfsm-reboot", NULL,
+                                      XFSM_SHUTDOWN_RESTART, dialog);
 
   gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
   gtk_widget_set_sensitive (button, can_restart);
@@ -262,8 +269,8 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
     }
 
   button = xfsm_logout_dialog_button (_("Shut _Down"), "system-shutdown",
-                                      "xfsm-shutdown", XFSM_SHUTDOWN_SHUTDOWN,
-                                      dialog);
+                                      "xfsm-shutdown", NULL,
+                                      XFSM_SHUTDOWN_SHUTDOWN, dialog);
 
   gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
   gtk_widget_set_sensitive (button, can_shutdown);
@@ -271,6 +278,7 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
 
   /* new row for suspend/hibernate/hybrid sleep */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, BORDER);
+  gtk_box_set_homogeneous (GTK_BOX (hbox), TRUE);
   gtk_box_pack_start (GTK_BOX (button_vbox), hbox, FALSE, TRUE, 0);
 
   /**
@@ -285,8 +293,8 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
           if (can_suspend)
             {
               button = xfsm_logout_dialog_button (_("Sus_pend"), "system-suspend",
-                                                  "xfsm-suspend", XFSM_SHUTDOWN_SUSPEND,
-                                                  dialog);
+                                                  "xfsm-suspend", NULL,
+                                                  XFSM_SHUTDOWN_SUSPEND, dialog);
 
               gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
               gtk_widget_set_sensitive (button, auth_suspend);
@@ -319,8 +327,8 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
           if (can_hibernate)
             {
               button = xfsm_logout_dialog_button (_("_Hibernate"), "system-hibernate",
-                                                  "xfsm-hibernate", XFSM_SHUTDOWN_HIBERNATE,
-                                                  dialog);
+                                                  "xfsm-hibernate", NULL,
+                                                  XFSM_SHUTDOWN_HIBERNATE, dialog);
 
               gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
               gtk_widget_set_sensitive (button, auth_hibernate);
@@ -349,9 +357,9 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
         {
           if (can_hybrid_sleep)
             {
-              button = xfsm_logout_dialog_button (_("_Hybrid Sleep"), "system-hibernate",
-                                                  "xfsm-hibernate", XFSM_SHUTDOWN_HYBRID_SLEEP,
-                                                  dialog);
+              button = xfsm_logout_dialog_button (_("H_ybrid Sleep"), "system-suspend-hibernate",
+                                                  "system-hibernate", "xfsm-hibernate",
+                                                  XFSM_SHUTDOWN_HYBRID_SLEEP, dialog);
 
               gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
               gtk_widget_set_sensitive (button, auth_hybrid_sleep);
@@ -381,11 +389,11 @@ xfsm_logout_dialog_init (XfsmLogoutDialog *dialog)
           if (can_switch_user)
             {
               button = xfsm_logout_dialog_button (_("Switch _User"), "system-users",
-                                                  "system-users-symbolic", XFSM_SHUTDOWN_SWITCH_USER,
-                                                  dialog);
+                                                  "system-users-symbolic", NULL,
+                                                  XFSM_SHUTDOWN_SWITCH_USER, dialog);
 
               gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
-              gtk_widget_set_sensitive (button, auth_hibernate);
+              gtk_widget_set_sensitive (button, can_switch_user);
               gtk_widget_show (button);
 
               gtk_widget_show (hbox);
@@ -490,16 +498,14 @@ static GtkWidget *
 xfsm_logout_dialog_button (const gchar      *title,
                            const gchar      *icon_name,
                            const gchar      *icon_name_fallback,
+                           const gchar      *icon_name_fallback2,
                            XfsmShutdownType  type,
                            XfsmLogoutDialog *dialog)
 {
   GtkWidget    *button;
   GtkWidget    *vbox;
-  GdkPixbuf    *pixbuf;
   GtkWidget    *image;
   GtkWidget    *label;
-  static gint   icon_size = 0;
-  gint          w, h;
   gint         *val;
   GtkIconTheme *icon_theme;
 
@@ -518,24 +524,17 @@ xfsm_logout_dialog_button (const gchar      *title,
   gtk_container_add (GTK_CONTAINER (button), vbox);
   gtk_widget_show (vbox);
 
-  if (G_UNLIKELY (icon_size == 0))
-    {
-      if (gtk_icon_size_lookup (GTK_ICON_SIZE_DND, &w, &h))
-        icon_size = MAX (w, h);
-      else
-        icon_size = 32;
-    }
+  icon_theme = gtk_icon_theme_get_default ();
 
-  icon_theme = gtk_icon_theme_get_for_screen (gtk_window_get_screen (GTK_WINDOW (dialog)));
-  pixbuf = gtk_icon_theme_load_icon (icon_theme, icon_name, icon_size, 0, NULL);
-  if (G_UNLIKELY (pixbuf == NULL))
-    {
-      pixbuf = gtk_icon_theme_load_icon (icon_theme, icon_name_fallback,
-                                         icon_size, GTK_ICON_LOOKUP_GENERIC_FALLBACK,
-                                         NULL);
-    }
+  image = gtk_image_new ();
+  if (gtk_icon_theme_has_icon (icon_theme, icon_name))
+      gtk_image_set_from_icon_name (GTK_IMAGE (image), icon_name, GTK_ICON_SIZE_DND);
+  else if (gtk_icon_theme_has_icon (icon_theme, icon_name_fallback))
+      gtk_image_set_from_icon_name (GTK_IMAGE (image), icon_name_fallback, GTK_ICON_SIZE_DND);
+  else if (gtk_icon_theme_has_icon (icon_theme, icon_name_fallback2))
+      gtk_image_set_from_icon_name (GTK_IMAGE (image), icon_name_fallback2, GTK_ICON_SIZE_DND);
 
-  image = gtk_image_new_from_pixbuf (pixbuf);
+  gtk_image_set_pixel_size (GTK_IMAGE (image), 32);
   gtk_box_pack_start (GTK_BOX (vbox), image, FALSE, FALSE, 0);
   gtk_widget_show (image);
 
@@ -822,8 +821,6 @@ xfsm_logout_dialog (const gchar      *session_name,
                              "type", GTK_WINDOW_POPUP,
                              "screen", screen, NULL);
 
-      xfsm_window_add_border (GTK_WINDOW (dialog));
-
       gtk_widget_realize (dialog);
       gdk_window_set_override_redirect (gtk_widget_get_window (dialog), TRUE);
       gdk_window_raise (gtk_widget_get_window (dialog));
